#ifndef INCL_get_subst_matrix_H
#define INCL_get_subst_matrix_H
#include <iostream>
#include "tigr++/TigrCommandLine.H"
#include "tigr++/TigrFastaReader.H"
#include "tigr++/TigrTriangMatrix.H"
#include "tigr++/TigrArray1D.H"
#include "tigr++/TigrConfigFile.H"
#include "tigr++/TigrProteinTrans.H"
#include "tigr++/TigrRandom.H"
#include "tigr++/TigrExceptions.H"
#include "tigr++/AminoAlphabet.H"
#include "tigr++/DnaAlphabet.H"
#include "tigr++/TigrTime.H"
#include "SubstitutionMatrix.H"
#include "Needleman.H"
#include "exonomy/GraphBuilder.H"
#include "exonomy/GraphReducer.H"
#include "exonomy/PseudoStopDetector.H"

const int LOCAL_OPTIMALITY_DISTANCE=30;
const int NUM_ORF_PAIRS=300;

typedef TigrVector<ParseEdge*> ORF;
typedef TigrVector<ParseVertex*> VertexList;

class Application
{
public:
  Application();
  int main(int argc,char *argv[]);
private:
  bool shouldQuantize;
  TigrTriangMatrix<double> *matrix;
  TigrArray1D<double> *marginals;
  double gapOpenPenalty, gapExtensionPenalty;
  double randomOpenPenalty, randomExtensionPenalty;
  TigrVector<SignalDetector*> signalDetectors;
  GraphBuilder *graphBuilder;
  GraphReducer reducer;
  TigrString infile1, infile2;

  void processOrthologues(const TigrString &proteinFile1,
			  const TigrString &proteinFile2,
			  SubstitutionMatrix &blosumMatrix,
			  double blosumGapOpen,double blosumGapExtend);
  void processDNA(const TigrString &dnaFile1,const TigrString &dnaFile2,
		  SubstitutionMatrix &blosumMatrix,
		  double blosumGapOpen,double blosumGapExtend);
  void produceOutput();
  TigrVector<Sequence*> *loadSequences(const TigrString &filename,
				       Alphabet &);
  void deleteSequences(TigrVector<Sequence*> *);
  void updateCounts(Alignment &,int &gapOpenings,int &gapExtensions,
		    int &totalPairs);
  void initGraphBuilder(TigrConfigFile &configFile);
  void addSignalDetector(TigrString modelTag,
			 ModelLoader &loader,
			 SignalType signalType,
			 TigrString consensusTag,
			 TigrConfigFile &configFile,
			 SignalDetector *&detector,
			 SignalDetector *&minusDetector);
  void getMarginals(Alignment &alignment,TigrArray1D<double> &marginals,
		    int &numGapOpenings,int &numGapExtensions,int &sampleSize,
		    int &indels);
  void computeLogOdds();
  void initMatrix();
  ORF *sampleORFs(VertexList &,int numVertices);
  Sequence *getProtein(ORF &,const TigrString &substrate);
  VertexList *getRightEnds(ParseGraph &);
};


#endif
