
// ungapped-aligner.C

#include <string>
#include <iostream>
#include <math.h>
#include "tigr++/TigrCommandLine.H"
#include "tigr++/TigrFastaReader.H"
#include "SubstitutionMatrix.H"
#include "tigr++/AminoAlphabet.H"
#include "tigr++/DnaAlphabet.H"
#include "UngappedAligner.H"


class Application
{
public:
  Application();
  int main(int argc,char *argv[]);
};


int main(int argc,char *argv[])
  {
    try
      {
	Application app;
	return app.main(argc,argv);
      }
    catch(const char *p)
      {
	cerr << p << endl;
      }
    catch(const string &msg)
      {
	cerr << msg.c_str() << endl;
      }
    catch(const exception &e)
      {
	cerr << "STL exception caught in main:\n" << e.what() << endl;
      }
    catch(...)
      {
	cerr << "Unknown exception caught in main" << endl;
      }
    return -1;
  }



Application::Application()
  {
    // ctor
  }



int Application::main(int argc,char *argv[])
  {
    // Process command line
    TigrCommandLine cmd(argc,argv,"");
    if(cmd.numArgs()!=4)
      throw string("ungapped-aligner <SubstitutionMatrix> <*.fasta> <*.fasta> DNA|PROTEIN\n\n\
example: ungapped-aligner blosum62 5 2 1.fasta 2.fasta DNA|PROTEIN\n");
    TigrString matrixFile=cmd.arg(0);
    TigrString file1=cmd.arg(1);
    TigrString file2=cmd.arg(2);
    TigrString type=cmd.arg(3);
    if(type!="DNA" && type!="PROTEIN") throw "specify DNA or PROTEIN";

    Alphabet &alphabet=
      (type=="DNA") ? 
      static_cast<Alphabet&>(DnaAlphabet::global) : 
      static_cast<Alphabet&>(AminoAlphabet::global);
    SubstitutionMatrix<float> M(matrixFile,alphabet);
    Sequence *seq1=Sequence::load(file1,alphabet);
    Sequence *seq2=Sequence::load(file2,alphabet);

    UngappedAligner<float> aligner(alphabet,*seq1,*seq2,M);
    float score=aligner.getScore();
    cout<<"score="<<score<<endl;
    return 0;
  }

